@@ -1,10 +1,10 @@
 #!/usr/bin/perl -w
 
-# $Id: clamav-update.pl,v 1.22 2007/03/17 17:15:35 okamura Exp $
+# $Id: clamav-update.pl,v 1.38 2007/07/14 21:09:07 okamura Exp $
 
 =head1 NAME
 
-clamav-update.pl - auto update clamav
+clamav-update.pl - auto updater for ClamAV
 
 =head1 SYNOPSIS
 
@@ -14,10 +14,44 @@ clamav-update.pl - auto update clamav
 
 =head1 DESCRIPTION
 
-B<clamav-update.pl> update clamav for ClamXav user.
+B<clamav-update.pl> is automatic customizable updater for general software.
+Mainly, It is developed to update Clam AntiVirus.
+Default configuration is designed for ClamAV Engine for ClamXav user
+on Mac OS X.
 
-But it isn't only for them.
-See http://clamav-update.sourceforge.jp/docs/clamav-update/index.html
+B<clamav-update.pl> acts as:
+
+1. Check execution situation for security.
+
+Only root can execute B<clamav-update.pl>.
+If someone execute it with other user's privilege, it show warning and quit.
+Don't add setuid to it.
+If do so, it can not be executed.
+
+2. Analizes arguments and loads configurations.
+
+Default configuration file is F</usr/local/clamXav/etc/clamav-update.conf>.
+
+3. Compares newest version and current version of target.
+
+B<clamav-update.pl> will continue
+if and only if newest version is newer than current version.
+The method of getting the version strings and the method of comparing them
+are defined by the configuration.
+
+4. Executes phases defined on configuration files.
+
+If errors are detected in any phase, execute commands set defined as roll back
+in that phase. and return error.
+
+In other words, we assumes only followings.
+
+  + root uses clamav-update.pl.
+  + clamav-update.pl updates target software,
+    if newer version of it is released. 
+
+Method of updating is given from
+your configuration file or command line options.
 
 =cut
 
@@ -112,28 +146,8 @@ sub _Fdlog {
 
 sub IsMasked {
 	my	($priority) = @_;
-	my	$numpri = undef;
-	my	$numfac = undef;
 
-	foreach (split(/\W+/, $priority, 2)) {
-		my	$num = Sys::Syslog::xlate($_);
-
-		if (/^kern$/ or $num < 0) {
-			croak "invalid level/facility: $_";
-		}
-		elsif ($num <= &Sys::Syslog::LOG_PRIMASK) {
-			croak "too many levels given: $_"	if defined($numpri);
-			$numpri = $num;
-			return 1	unless (
-				Sys::Syslog::LOG_MASK($numpri) & $Sys::Syslog::maskpri);
-		}
-		else {
-			croak "too many facilities given: $_" if defined($numfac);
-			$numfac = $num;
-		}
-	}
-
-	return 0;
+	return setlogmask(0) < Sys::Syslog::LOG_UPTO(Sys::Syslog::xlate($priority));
 }
 
 sub SetEnv {
@@ -187,7 +201,11 @@ sub PrepareLogging {
 
 	# setlogsock
 	if (ref($Setting{logging}->{setlogsock}) eq 'GLOB') {
+		my	$currentSelected = select $Setting{logging}->{setlogsock};
+
 		*Log = *_Fdlog;
+		$| = 1;
+		select $currentSelected;
 	}
 	else {
 		if (ref($Setting{logging}->{setlogsock}) eq 'ARRAY') {
@@ -215,6 +233,15 @@ sub PrepareLogging {
 	}
 }
 
+sub CloseLog {
+	if ($SyslogOpened) {
+		closelog();
+	}
+	elsif (*Log eq *_Fdlog) {
+		close $Setting{logging}->{setlogsock};
+	}
+}
+
 sub LoadConfig {
 	my	($path) = @_;
 
@@ -591,10 +618,10 @@ sub PhaseSpecifier4General {
 # ClamAV for ClamXav
 #	1. download gzip compressed tar file.
 #	2. extract it.
-#	3. do ./configure --prefix=/usr/local
+#	3. do ./configure --prefix=/usr/local/clamXav
 #	4. do make
 #	5. do make install
-#	6. change owner, group, permission
+#	6. change owner, group and permission
 sub PhaseSpecifier4ClamAV4ClamXav {
 	#### Initialization ####
 	# use PhaseSpecifier4General as base.
@@ -612,15 +639,15 @@ sub PhaseSpecifier4ClamAV4ClamXav {
 	push @{$Phase{install}->{method}}, (
 		[qw(install -d -o root -g admin -m 0775 /usr/local/clamXav)],
 		[qw(chown -R root:admin /usr/local/clamXav/etc)],
-		[qw(find /usr/local/clamXav/etc -type d -exec chmod 0775 {} \;)],
-		[qw(find /usr/local/clamXav/etc -type f -exec chmod 0664 {} \;)],
+		[qw(find /usr/local/clamXav/etc -type d -exec chmod 0775 {} ;)],
+		[qw(find /usr/local/clamXav/etc -type f -exec chmod 0664 {} ;)],
 		[qw(chown -R root:admin /usr/local/clamXav/bin)],
 		[qw(chmod -R 0755 /usr/local/clamXav/bin)],
 		[qw(chown clamav /usr/local/clamXav/bin/freshclam)],
 		[qw(chmod u+s /usr/local/clamXav/bin/freshclam)],
 		[qw(chown -R clamav:clamav /usr/local/clamXav/share/clamav)],
-		[qw(find /usr/local/clamXav/share/clamav -type d -exec chmod 0775 {} \;)],
-		[qw(find /usr/local/clamXav/share/clamav -type f -exec chmod 0664 {} \;)],
+		[qw(find /usr/local/clamXav/share/clamav -type d -exec chmod 0775 {} ;)],
+		[qw(find /usr/local/clamXav/share/clamav -type f -exec chmod 0664 {} ;)],
 		[qw(touch /usr/local/clamXav/share/clamav/freshclam.log)],
 		[qw(chmod 0664 /usr/local/clamXav/share/clamav/freshclam.log)],
 	);
@@ -628,7 +655,7 @@ sub PhaseSpecifier4ClamAV4ClamXav {
 	1;
 }
 
-# pkg on dmg file
+# pkg on dmg file for Mac OS X
 #	1. download dmg file.
 #	2. attach it.
 #	3. install from pkg file.
@@ -795,7 +822,7 @@ sub PhaseSpecifier4ClamavUpdate {
 #-------------------------------------------------------------------------------
 # グローバル変数初期値
 #-------------------------------------------------------------------------------
-$Version = 'clamav-update.pl v2.1.1';
+$Version = 'clamav-update.pl v2.2.3';
 
 %ErrorCode = (
 	arguments		=> 1,
@@ -890,51 +917,112 @@ $SyslogOpened = 0;
 
 =item -h|--help
 
-Display help.
+Show manual of B<clamav-update.pl> to standard output.
+You can use perldoc instead of -h option to see manual.
+We recommend perldoc's one.
 
 =item -V|--version
 
-Display version.
+Show version of B<clamav-update.pl> to standard output
 
-=item -c|--config I<filepath>
+=item -c I<filepath> | --config I<filepath>
+| -c=I<filepath> | --config=I<filepath>
 
-Set the configuration file to I<filepath>.
-DEFAULT: /usr/local/clamXav/etc/clamav-update.conf
+Read config_file as configuration file.
+Default value is F</usr/local/clamXav/etc/clamav-update.conf>.
 
 =item -f|--force
 
-Force install latest version.
+Force to install. Change string of current version of the software to empty.
 
-=item -s I<donwload_from> | --src I<donwload_from>
+This is for forcing to install but is depends of how to get latest version info
+and definition of comparing version function.
+This correspond to C<$Setting{option}-E<gt>{force}>.
 
-Change download base URL to I<donwload_from>.
-DEGAULT: http://jaist.dl.sourceforge.net/sourceforge/clamav
+=item -s I<download_from> | --src I<download_from>
+-s=I<download_from> | --src=I<download_from>
+
+Specify base URL as I<download_from> for downloading target software.
+Default value is F<http://osdn.dl.sourceforge.jp/clamav-update>.
+
+B<clamav-update.pl> DO NOT use this value directly.
+This value is used in some defined phase.
+This correspond to C<$Setting{option}-E<gt>{src}>.
 
 =item -d I<download_to> | --dst I<download_to>
+-d=I<download_to> | --dst=I<download_to>
 
-Change directory of destination to I<donwload_from>.
-DEFAULT: Random named directory under system temporary directory.
+Specify a temporary directory to save download file as I<download_to>.
+Default value is some alphanumeric strings at random
+in your system temporary directory.
+Default directory will be removed before clamav-update.pl finish.
+
+B<clamav-update.pl> DO NOT use this value directly.
+This value is used in some defined phase. 
+This correspond to C<$Setting{option}-E<gt>{dst}>.
 
 =item -n I<package_name> | --name I<package_name>
+ -n=I<package_name> | --name=I<package_name>
 
-Change package name to I<package_name>.
-DEFAULT: clamav
+Specify I<package_name> as package name of software.
+Default value is clamav.
+
+B<clamav-update.pl> DO NOT use this value directly.
+This value is used in some defined phase.
+This correspond to C<$Setting{option}-E<gt>{name}>.
 
 =item -e I<package_extention> | --ext I<package_extention>
+ -e=I<package_extention> | --ext=I<package_extention>
+ 
+Specify I<package_extention> as suffix of an archive of the software.
+Default value is tar.gz .
 
-Change package file extention to I<package_extention>.
-DEFAULT: tar.gz
+B<clamav-update.pl> DO NOT use this value directly.
+This value is used in some defined phase.
+This correspond to C<$Setting{option}-E<gt>{ext}>.
 
-=item --limit-version I<version_number>
+=item --limit-version I<version_number> | --limit-version=I<version_number>
 
-Change version that is limmited update to I<version_number>.
-DEFAULT: NONE
+Specify upper limit of version I<version_number>.
+B<clamav-update.pl> doesn't update I<version_number> or higher.
+Default value is undefined. It means unlimited.
 
-=item --limit-action I<phase_name>
+This correspond to C<$Setting{option}-E<gt>{limit}-E<gt>{version}>.
 
-Change phase name for actions when update is limited to I<phase_name>.
-DEFAULT: NONE
+=item --limit-action I<phase_name> | --limit-action=I<phase_name>
 
+If B<clamav-update.pl> don't update software because of
+its upper limit of version, B<clamav-update.pl> execute phase.
+Default value is undefined. It means no action.
+
+=item --option I<name>=I<value> | --option=I<name>=I<value>
+
+Change value of C<$Setting{option}-E<gt>{I<name>}> to I<value>.
+This treats as changing configuration file temporally.
+If slash "/" character is included in name,
+name is treated as path.
+For example:
+
+  clamav-update.pl --option one/two/three=four ...
+
+It means that
+
+  $Setting{option}->{one}->{two}->{three} = 'four';
+
+This option configuration is overwrote by other directly option.
+For example, if you execute following command line:
+
+  clamav-update.pl --src download_from1 --option src=download_from2 ...
+
+B<clamav-update.pl> download from I<download_from1>.
+Because this is equiv to situation that
+you have bellow line in configuration file:
+
+  $Setting{option}->{src} = 'download_from2';
+
+and execute following command line:
+
+  clamav-update.pl --src download_from1 ...
 
 =back
 
@@ -963,6 +1051,24 @@ EOF
 	'e|ext=s'			=> \$Option{ext},
 	'limit-version=s'	=> \$Option{limit}->{version},
 	'limit-action=s'	=> \$Option{limit}->{action},
+	'option=s'			=> sub {
+		my	($namePath, $value) = split(/=/, $_[1], 2);
+		my	@namePath = split(/\//, $namePath);
+		my	$cur = $Setting{option};
+		my	$last = pop @namePath;
+		my	$history = '$Setting{option}';
+
+		foreach my $name (@namePath) {
+			$history .= '->{\''.$name.'\'}';
+			$cur->{$name} = {}	unless (exists $cur->{$name});
+			unless (ref($cur->{$name}) eq 'HASH') {
+				printf STDERR "%s isn't a hash reference.\n", $history;
+				exit $ErrorCode{arguments};
+			}
+			$cur = $cur->{$name};
+		}
+		$cur->{$last} = $value;
+	},
 ) or pod2usage($ErrorCode{arguments});
 
 
@@ -1061,18 +1167,48 @@ if (&{$Setting{version}->{compare}}($CurrentVersion, $LatestVersion) < 0) {
 # 終了処理
 END {
 	Log 'info', 'Finish clamav-update.pl --config %s', GetOptionValue('config');
-	closelog()	if ($SyslogOpened);
+	CloseLog();
 }
 
 #===============================================================================
 
+=head1 SEE ALSO
+
+Reference of Configuration: L<http://clamav-update.sourceforge.jp/docs/clamav-update-2/chapter2.html#B-3>
+
 =head1 CHANGE LOG
 
 =over 4
 
+=item v2.2.3
+
+ * Update POD based fu7mu4's translation.
+ * Update english comments based fu7mu4's translation.
+
+=item v2.2.2
+
+ * The problem that warnning messages of Perl are outputted
+   when the version of SYSLOG is new is fixed.
+
+=item v2.2.1
+
+ * When log destination is GLOB, restrain the buffering.
+ * When log destination is GLOB, close it explicitly.
+ * Argument of find command in the install phase of
+   PhaseSpecifier4ClamAV4ClamXav was fixed.
+
+=item v2.2
+
+ * Add option named "option"
+
+=item v2.1.1
+
+ * install.sh problems are fixed.
+ * All v1.x setting description are changed for v2.x
+
 =item v2.1
 
- * Add new feature "update limitting". 
+ * Add new feature "update limitting".
  * Add new optinos description to POD.
  * Improve build-in phase specifier functions.
    Now you can use PhaseSpecifier4ClamavUpdate on the system which use GNU
@@ -1119,6 +1255,8 @@ Initial version.
 
 OKAMURA Yuji E<lt>okamura@users.sourceforge.jpE<gt>
 
+Manual translator: fu7mu4
+
 =cut
 
 __END__;
