@@ -515,7 +515,7 @@ fi
 
 
 #### Paramaters Check ####
-if expr "$Prefix" : '/' > /dev/null; then
+if expr "$Prefix" : '/.*' > /dev/null; then
 	Prefix=`normalizeDir "$Prefix"`
 else
 	echo "Use absolute path for --prefix option: $Prefix" 1>&2
@@ -532,8 +532,8 @@ elif [ \! \( -d "$Prefix/bin" -a -d "$Prefix/etc" \) ]; then
 	exit 3
 fi
 
-if expr "$ClamavPrefix" : '/' > /dev/null; then
-	ClamavPrefix=`normalizeDir "$Prefix"`
+if expr "$ClamavPrefix" : '/.*' > /dev/null; then
+	ClamavPrefix=`normalizeDir "$ClamavPrefix"`
 else
 	echo "Use absolute path for --clamav-prefix option: $ClamavPrefix" 1>&2
 	exit 3
